@@ -12,9 +12,7 @@ from pyquery import PyQuery
 from lxml import etree
 
 
-from google_parser.exceptions import EmptySerp, NoBodyInResponseError, BadGoogleParserError, \
-    SnippetsParserException, BadUrlError
-
+from google_parser.exceptions import SnippetsParserException, GoogleParserError
 
 __all__ = ['GoogleParser']
 
@@ -221,6 +219,9 @@ class GoogleParser(object):
 
     def __init__(self, content, xhtml_snippet=False, snippet_fields=('d', 'p', 'u', 't', 's', 'm')):
         self.content = to_unicode(content)
+        # with open('go.html', 'w') as f:
+        #     f.write(self.content)
+
         self.xhtml_snippet = xhtml_snippet
         self.snippet_fields = snippet_fields
 
@@ -294,7 +295,7 @@ class GoogleParser(object):
     def get_context_snippet_title(self, content):
         res = re.search(ur'<h3>\s*<a[^>]+?></a>\s*<a[^>]*?href="([^"]+?)"[^>]*?>\s*(.*?)\s*</a>\s*</h3>', content, re.I | re.M | re.S)
         if not res:
-            raise BadGoogleParserError(u'Не удалось распарсить тайтл в сниппете: {0}'.format(content))
+            raise GoogleParserError(u'Не удалось распарсить тайтл в сниппете: {0}'.format(content))
         return {'u': SnippetsParserDefault.format_context_link(res.group(1)), 't': res.group(2)}
 
     def get_context_visible_url(self, content):
@@ -325,7 +326,17 @@ class GoogleParser(object):
         snippets = self.get_snippets()
 
         if not snippets:
-            raise EmptySerp()
+            raise GoogleParserError('not found snippets')
+
+        for pos, snippet in enumerate(snippets, start=1):
+            if snippet['p'] != pos:
+                raise GoogleParserError('bad position')
+
+            if not snippet['u']:
+                raise GoogleParserError('bad url')
+
+            if not snippet['d']:
+                raise GoogleParserError('bad domain')
 
         return {'pc': pagecount, 'sn': snippets}
 
@@ -357,6 +368,7 @@ class GoogleParser(object):
         pagecount = 0
         patterns = (ur'<div[^>]+resultStats(?:[^>]+)?>Результатов: примерно (.*?)<',
         ur'<div[^>]+resultStats(?:[^>]+)?>(.*?)<nobr>',
+        ur'<div[^>]+result-stats(?:[^>]+)?>(.*?)<nobr>',
         ur'<div[^>]+resultStats(?:[^>]+)?>Результатов:(.*?)</div>',
         ur'<div>Результатов:\s*(.*?)</div>',
         ur'Результатов:\s*(.*?),.*?</div>',
@@ -378,12 +390,11 @@ class GoogleParser(object):
     def get_snippets(self):
         res = re.compile('<body.*?</body>', re.DOTALL).search(self.content)
         if not res:
-            raise NoBodyInResponseError()
+            raise GoogleParserError('no body in response')
 
-        # with open('go.html', 'w') as f:
-        #     f.write(self.content)
-
-        if re.search('<div class="med[^"]*?" id="res" role="main">', self.content):
+        if re.search('<div class="[^"]*?" id="res" role="main">.*?<div id="bottomads"', self.content, flags=re.S):
+            return SnippetsParserAfter_2021_01_29(self.snippet_fields).get_snippets(self.content)
+        elif re.search('<div class="[^"]*?" id="res" role="main">', self.content):
             return SnippetsParserAfter_2016_03_10(self.snippet_fields).get_snippets(self.content)
         elif '<body jsmodel="' in self.content:
             return MobileSnippetsParser(self.snippet_fields).get_snippets(self.content)
@@ -391,13 +402,14 @@ class GoogleParser(object):
             return SnippetsParserDefault(self.snippet_fields).get_snippets(self.content)
         elif '<div id="search"><div id="ires">' in self.content:
             return SnippetsParserUnil_2015_07_23(self.snippet_fields).get_snippets(self.content)
-        raise BadGoogleParserError()
+        raise GoogleParserError('not found parser version')
 
     def is_not_found(self):
         patterns = [
-            re.compile(ur'>\s*По\s*запросу\s*<(?:em|b)>.*?</(?:em|b)>\s*ничего\s*не\s*найдено\.', re.I | re.M | re.S),
+            re.compile(ur'>\s*По\s*запросу\s*<(?:em|b|span)>.*?</(?:em|b|span)>\s*ничего\s*не\s*найдено\.', re.I | re.M | re.S),
             re.compile(ur'Извините,\s*у\s*нас\s*нет\s*информации\s*об\s*адресе\s*<(?:em|b)>.*?</(?:em|b)>', re.I | re.M | re.S),
             re.compile(ur'<div id="sbfrm_l"></div>', re.I | re.M | re.S),
+            re.compile(ur'Результатов: примерно 0', re.I | re.M | re.S),
         ]
         res = False
         for pattern in patterns:
@@ -416,7 +428,7 @@ class SnippetsParserDefault(object):
     def get_snippets(self, body):
         res = self.result_regexp.findall(body)
         if not res:
-            raise BadGoogleParserError()
+            raise GoogleParserError('no body in response')
 
         result = []
         position = 0
@@ -440,7 +452,7 @@ class SnippetsParserDefault(object):
                         raise
 
                 # игнорим сниппет с картинками
-                if self._is_map_snippet(item['u']) or item['u'].startswith('/search'):
+                if self._is_map_snippet(item['u']) or item['u'].startswith('/search') or item['u'].startswith('/images'):
                     position -= 1
                     continue
 
@@ -491,7 +503,7 @@ class SnippetsParserDefault(object):
         try:
             return get_full_domain_without_scheme(url)
         except UnicodeError as e:
-            raise BadUrlError(u'некорректный урл: {0}'.format(url))
+            raise GoogleParserError(u'некорректный урл: {0}'.format(url))
 
     def _is_empty_snippet(self, snippet):
         return '<h3 class="r"></h3>' in snippet
@@ -547,7 +559,7 @@ class SnippetsParserDefault(object):
         res = re.compile(ur'<div>(.*?)</div>', re.I | re.M | re.S).search(snippet)
         if res:
             return SnippetsParserDefault.strip_tags(res.group(1))
-        raise BadGoogleParserError(u'не удалось найти описание сниппета: {}'.format(snippet))
+        raise GoogleParserError(u'не удалось найти описание сниппета: {}'.format(snippet))
 
     def _parse_description_snippet(self, snippet):
         res = re.compile(ur'<span class="(?:st|aCOpRe)">(.*?)</span>\s*(?:<br>|</div>|<div|</a>)', re.I | re.M | re.S).search(snippet)
@@ -573,7 +585,7 @@ class MobileSnippetsParser(SnippetsParserDefault):
             snippet
         )
         if not match:
-            raise BadGoogleParserError(snippet)
+            raise GoogleParserError(snippet)
 
         url = HTMLParser().unescape(match.group(1))
         title = HTMLParser().unescape(
@@ -624,7 +636,7 @@ class MobileSnippetsParser(SnippetsParserDefault):
         dom = PyQuery(body)
         rso_div = dom('#rso')
         if not rso_div:
-            raise BadGoogleParserError()
+            raise GoogleParserError()
 
         divs = rso_div.find('div.mnr-c')
         serp = []
@@ -675,19 +687,19 @@ class SnippetsParserUnil_2015_07_23(SnippetsParserDefault):
 
 class SnippetsParserAfter_2016_03_10(SnippetsParserDefault):
     snippets_regexp = re.compile(ur'(<div class=(?:"g\s*(?:card-section)?"(?: data-hveid="[^"]+?")?(?: data-ved="[^"]+?")?>|"g\s+[^_"]*"[^>]*>)\s*(?:<h2[^>]+?>[^<]+?</h2>)?\s*(?:<div data-hveid="[^"]+?">|<div>)?<!--m-->\s*.*?</div><!--n-->\s*(?:</div>))', re.I | re.M | re.S)
-    result_regexp = re.compile(ur'(<div class="med[^"]*?" id="res" role="main">.*?<!--z-->)', re.I | re.M | re.S)
+    result_regexp = re.compile(ur'(<div class="[^"]*?" id="res" role="main">.*?<!--z-->)', re.I | re.M | re.S)
 
     def get_snippets(self, body):
         res = self.result_regexp.findall(body)
         if not res:
-            raise BadGoogleParserError()
+            raise GoogleParserError('no body in response')
 
         result = []
         position = 0
         for body in res:
             snippets = self.snippets_regexp.findall(body)
             for snippet in snippets:
-                if re.search(ur'class="g[^"]+\s+obcontainer\s*', snippet):
+                if re.search(ur'class="g[^"]+obcontainer\s*', snippet):
                     continue
 
                 position += 1
@@ -707,6 +719,83 @@ class SnippetsParserAfter_2016_03_10(SnippetsParserDefault):
 
                 result.append(item)
 
+        if len(result) >= 2 and result[0]['u'] == result[1]['u']:
+            result = result[1:]
+            for pos, snippet in enumerate(result, start=1):
+                snippet['p'] = pos
+
+        return result
+
+
+class SnippetsParserAfter_2021_01_29(SnippetsParserAfter_2016_03_10):
+    snippets_regexp = None
+    result_regexp = re.compile(ur'(<div class="[^"]*?" id="res" role="main">.*?<div id="bottomads")', re.I | re.M | re.S)
+
+    def get_snippet(self, position, snippet):
+        title, url = self._parse_title_snippet(snippet, position)
+        return {
+            'p': position,
+            'u': url,
+            'd': self._get_domain(url),
+            'm': self._is_map_snippet(url),
+            't': self._get_title(title),
+            's': self._get_descr(snippet, url),
+            'h': self._get_html(snippet),
+            'vu': self._get_vu(snippet),
+        }
+
+    def get_snippets(self, body):
+        res = self.result_regexp.findall(body)
+        if not res:
+            raise GoogleParserError('no body in response')
+
+        result = []
+        position = 0
+        for body in res:
+            dom = PyQuery(body)
+            snippets = dom('div.g')
+            for snippet in snippets:
+                html = HTMLParser().unescape(
+                    etree.tostring(snippet)
+                )
+
+                if len(snippet.cssselect('div.g')) > 1:
+                    continue
+
+                if re.search(ur'class="g[^"]+(?:obcontainer|g-blk)\s*', html):
+                    continue
+
+                # дополнительные элементы
+                if re.search(ur'class="[^"]+?__outer-card', html):
+                    continue
+
+                # дополнительные элементы
+                if re.search(ur'class="[^"]*?lu_map_section', html):
+                    continue
+
+                if re.search(ur'id="imagebox_bigimages"', html):
+                    continue
+
+                if re.search(ur'<div\s*class="[^"]*rrec[^"]*"', html):
+                    continue
+
+                position += 1
+                try:
+                    item = self.get_snippet(position, html)
+                except SnippetsParserException:
+                    if self._is_empty_snippet(snippet):
+                        position -= 1
+                        continue
+                    else:
+                        raise
+
+                # игнорим сниппет с картинками и карты
+                if self._is_map_snippet(item['u']) or item['u'].startswith('/search'):
+                    position -= 1
+                    continue
+
+                result.append(item)
+
         if len(result) >= 2 and result[0]['u'] == result[1]['u']:
             result = result[1:]
 
@@ -763,11 +852,11 @@ class GoogleMobileParser(GoogleParser):
     def get_snippets(self):
         res = re.compile('<body.*?</body>', re.DOTALL).search(self.content)
         if not res:
-            raise NoBodyInResponseError()
+            raise GoogleParserError('no body in response')
 
         if '<div class="srg"' in self.content:
             return SnippetsMobileParserDefault(self.snippet_fields).get_snippets(self.content)
-        raise BadGoogleParserError()
+        raise GoogleParserError()
 
 
     @classmethod
